﻿using Luski.net.Enums;
using Luski.net.Interfaces;
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text.Json.Serialization;
using System.Text.Json.Serialization.Metadata;
using System.Threading.Tasks;

namespace Luski.net.JsonTypes.BaseTypes;

[Browsable(false)]
[EditorBrowsable(EditorBrowsableState.Never)]
public abstract class SocketUserBase : IncomingHTTP, IUser
{
    [JsonPropertyName("id")]
    [JsonInclude]
    public long Id { get; internal set; } = default!;
    [JsonPropertyName("username")]
    [JsonInclude]
    public string Username { get; internal set; } = default!;
   // [JsonPropertyName("tag")]
    //[JsonInclude]
    //public short Tag { get; internal set; } = default!;
    [JsonPropertyName("status")]
    [JsonInclude]
    public UserStatus Status { get; internal set; } = default!;
    [JsonPropertyName("picture_type")]
    [JsonInclude]
    public PictureType PictureType { get; internal set; } = default!;

    [JsonPropertyName("flags")]
    [JsonInclude]
    public UserFlag Flags { get; internal set; } = default!;

    public async Task<byte[]> GetAvatar()
    {
        if (Server.Cache != null)
        {
            bool isc = System.IO.File.Exists($"{Server.Cache}/avatars/{Id}");
            if (!isc) await Server.GetFromServer($"socketuserprofile/Avatar/{Id}", $"{Server.Cache}/avatars/{Id}");
        }
        return System.IO.File.ReadAllBytes($"{Server.Cache}/avatars/{Id}");
    }

    public Task<string> GetUserKey()
    {
        if (Server._user is null) throw new Exception("you are not loged in");
        if (Id == Server._user.Id && Encryption.MyPublicKey is not null) return Task.FromResult(Encryption.MyPublicKey);
        string data = Server.GetFromServer($"Keys/GetUserKey/{Id}").Content.ReadAsStringAsync().Result;
        return Task.FromResult(data);
    }

    internal static async Task<TUser> GetUser<TUser>(long UserId, JsonTypeInfo<TUser> Json) where TUser : SocketUserBase, new()
    {
        TUser user;
        if (Server.poeople is null) Server.poeople = new();
        if (Server.poeople.Count > 0 && Server.poeople.Any(s => s.Id == UserId))
        {
            TUser temp = Server.poeople.Where(s => s is TUser && s.Id == UserId).Cast<TUser>().FirstOrDefault()!;
            if (temp is SocketRemoteUser && (temp as SocketRemoteUser)!.Channel == null)
            {
                foreach (SocketDMChannel chan in Server.chans.Where(s => s is SocketDMChannel).Cast<SocketDMChannel>())
                {
                    if (chan.Type == ChannelType.DM && chan.Id != 0 && chan.MemberIdList is not null)
                    {
                        if (chan.MemberIdList.Any(s => s == UserId)) (temp as SocketRemoteUser)!.Channel = chan;
                    }
                }
            }
            return temp!;
        }
        while (true)
        {
            if (Server.CanRequest)
            {
                user = await Server.GetFromServer("socketuser",
                    Json,
                    new System.Collections.Generic.KeyValuePair<string, string?>("id", UserId.ToString()));
                break;
            }
        }

        if (user is null) throw new Exception("Server did not return a user");
        if (Server.poeople.Count > 0 && Server.poeople.Any(s => s.Id == UserId))
        {
            foreach (IUser? p in Server.poeople.Where(s => s.Id == UserId))
            {
                Server.poeople.Remove(p);
            }
        }
        if (Server._user is not null && UserId != 0 && UserId != Server._user.Id)
        {
            foreach (SocketDMChannel chan in Server.chans.Where(s => s is SocketDMChannel).Cast<SocketDMChannel>())
            {
                if (chan.Id != 0 && chan.MemberIdList is not null)
                {
                    if (chan.MemberIdList.Any(s => s == UserId)) (user as SocketRemoteUser)!.Channel = chan;
                }
            }
        }
        Server.poeople.Add(user);
        return user;
    }
}

[JsonSerializable(typeof(SocketUserBase))]
[JsonSourceGenerationOptions(
    GenerationMode = JsonSourceGenerationMode.Default,
    PropertyNamingPolicy = JsonKnownNamingPolicy.Unspecified,
    WriteIndented = false,
    DefaultIgnoreCondition = JsonIgnoreCondition.Never)]
internal partial class SocketUserBaseContext : JsonSerializerContext
{

}
